package spm

import (
	"archive/tar"
	"compress/gzip"
	"fmt"
	"io"
	"os"
	"path/filepath"
	"strings"
)

// DecompressPackage determines the package format and decompresses it
func DecompressPackage(downloadDir, packageName, arch, osName, pkgType, release, version string) (string, error) {
	// 1) Construct the .tar.gz name
	expectedFileName := fmt.Sprintf(
		"%s@%s@%s@%s@%s@%s.tar.gz",
		packageName, arch, osName, pkgType, release, version,
	)
	packagePath := filepath.Join(downloadDir, expectedFileName)

	// Check that file exists
	if _, err := os.Stat(packagePath); os.IsNotExist(err) {
		return "", fmt.Errorf("package file not found: %s", packagePath)
	}

	// 2) Build the folder path (minus ".tar.gz")
	folderName := strings.TrimSuffix(expectedFileName, ".tar.gz")
	tempDir := GetTempDir() // e.g. C:\Users\<User>\AppData\Local\Temp\spm_temp_164326
	decompressDir := filepath.Join(tempDir, folderName)

	// Ensure the folder
	if err := os.MkdirAll(decompressDir, 0755); err != nil {
		return "", fmt.Errorf("failed to create decompressDir: %w", err)
	}

	// 3) Decompress everything into `decompressDir`
	if err := decompressTarGz(packagePath, decompressDir); err != nil {
		return "", fmt.Errorf("failed to decompress: %w", err)
	}

	// Return the folder path we used
	return decompressDir, nil
}

func decompressTarGz(srcFile, destDir string) error {
	f, err := os.Open(srcFile)
	if err != nil {
		return err
	}
	defer f.Close()

	gzr, err := gzip.NewReader(f)
	if err != nil {
		return err
	}
	defer gzr.Close()

	tarReader := tar.NewReader(gzr)
	for {
		header, err := tarReader.Next()
		if err == io.EOF {
			break
		}
		if err != nil {
			return err
		}

		outPath := filepath.Join(destDir, header.Name)
		switch header.Typeflag {
		case tar.TypeDir:
			if err := os.MkdirAll(outPath, os.FileMode(header.Mode)); err != nil {
				return err
			}
		case tar.TypeReg:
			outFile, err := os.Create(outPath)
			if err != nil {
				return err
			}
			_, err = io.Copy(outFile, tarReader)
			outFile.Close()
			if err != nil {
				return err
			}
		default:
			// huh
		}
	}
	return nil
}
